from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .ai_interaction_mentioned_identity_set import AiInteractionMentionedIdentitySet
    from .chat_message_from_identity_set import ChatMessageFromIdentitySet
    from .chat_message_mentioned_identity_set import ChatMessageMentionedIdentitySet
    from .chat_message_reaction_identity_set import ChatMessageReactionIdentitySet
    from .communications_identity_set import CommunicationsIdentitySet
    from .engagement_identity_set import EngagementIdentitySet
    from .identity import Identity
    from .share_point_identity_set import SharePointIdentitySet

@dataclass
class IdentitySet(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # Optional. The application associated with this action.
    application: Optional[Identity] = None
    # Optional. The device associated with this action.
    device: Optional[Identity] = None
    # The OdataType property
    odata_type: Optional[str] = None
    # Optional. The user associated with this action.
    user: Optional[Identity] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IdentitySet:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IdentitySet
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        try:
            child_node = parse_node.get_child_node("@odata.type")
            mapping_value = child_node.get_str_value() if child_node else None
        except AttributeError:
            mapping_value = None
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.aiInteractionMentionedIdentitySet".casefold():
            from .ai_interaction_mentioned_identity_set import AiInteractionMentionedIdentitySet

            return AiInteractionMentionedIdentitySet()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.chatMessageFromIdentitySet".casefold():
            from .chat_message_from_identity_set import ChatMessageFromIdentitySet

            return ChatMessageFromIdentitySet()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.chatMessageMentionedIdentitySet".casefold():
            from .chat_message_mentioned_identity_set import ChatMessageMentionedIdentitySet

            return ChatMessageMentionedIdentitySet()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.chatMessageReactionIdentitySet".casefold():
            from .chat_message_reaction_identity_set import ChatMessageReactionIdentitySet

            return ChatMessageReactionIdentitySet()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.communicationsIdentitySet".casefold():
            from .communications_identity_set import CommunicationsIdentitySet

            return CommunicationsIdentitySet()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.engagementIdentitySet".casefold():
            from .engagement_identity_set import EngagementIdentitySet

            return EngagementIdentitySet()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.sharePointIdentitySet".casefold():
            from .share_point_identity_set import SharePointIdentitySet

            return SharePointIdentitySet()
        return IdentitySet()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .ai_interaction_mentioned_identity_set import AiInteractionMentionedIdentitySet
        from .chat_message_from_identity_set import ChatMessageFromIdentitySet
        from .chat_message_mentioned_identity_set import ChatMessageMentionedIdentitySet
        from .chat_message_reaction_identity_set import ChatMessageReactionIdentitySet
        from .communications_identity_set import CommunicationsIdentitySet
        from .engagement_identity_set import EngagementIdentitySet
        from .identity import Identity
        from .share_point_identity_set import SharePointIdentitySet

        from .ai_interaction_mentioned_identity_set import AiInteractionMentionedIdentitySet
        from .chat_message_from_identity_set import ChatMessageFromIdentitySet
        from .chat_message_mentioned_identity_set import ChatMessageMentionedIdentitySet
        from .chat_message_reaction_identity_set import ChatMessageReactionIdentitySet
        from .communications_identity_set import CommunicationsIdentitySet
        from .engagement_identity_set import EngagementIdentitySet
        from .identity import Identity
        from .share_point_identity_set import SharePointIdentitySet

        fields: dict[str, Callable[[Any], None]] = {
            "application": lambda n : setattr(self, 'application', n.get_object_value(Identity)),
            "device": lambda n : setattr(self, 'device', n.get_object_value(Identity)),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
            "user": lambda n : setattr(self, 'user', n.get_object_value(Identity)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("application", self.application)
        writer.write_object_value("device", self.device)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_object_value("user", self.user)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .retrieval_entity_type import RetrievalEntityType
    from .retrieval_extract import RetrievalExtract
    from .search_resource_metadata_dictionary import SearchResourceMetadataDictionary
    from .sensitivity_label_info import SensitivityLabelInfo

@dataclass
class RetrievalHit(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The extracts property
    extracts: Optional[list[RetrievalExtract]] = None
    # The OdataType property
    odata_type: Optional[str] = None
    # The resourceMetadata property
    resource_metadata: Optional[SearchResourceMetadataDictionary] = None
    # The resourceType property
    resource_type: Optional[RetrievalEntityType] = None
    # The sensitivityLabel property
    sensitivity_label: Optional[SensitivityLabelInfo] = None
    # The webUrl property
    web_url: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RetrievalHit:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RetrievalHit
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RetrievalHit()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .retrieval_entity_type import RetrievalEntityType
        from .retrieval_extract import RetrievalExtract
        from .search_resource_metadata_dictionary import SearchResourceMetadataDictionary
        from .sensitivity_label_info import SensitivityLabelInfo

        from .retrieval_entity_type import RetrievalEntityType
        from .retrieval_extract import RetrievalExtract
        from .search_resource_metadata_dictionary import SearchResourceMetadataDictionary
        from .sensitivity_label_info import SensitivityLabelInfo

        fields: dict[str, Callable[[Any], None]] = {
            "extracts": lambda n : setattr(self, 'extracts', n.get_collection_of_object_values(RetrievalExtract)),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
            "resourceMetadata": lambda n : setattr(self, 'resource_metadata', n.get_object_value(SearchResourceMetadataDictionary)),
            "resourceType": lambda n : setattr(self, 'resource_type', n.get_enum_value(RetrievalEntityType)),
            "sensitivityLabel": lambda n : setattr(self, 'sensitivity_label', n.get_object_value(SensitivityLabelInfo)),
            "webUrl": lambda n : setattr(self, 'web_url', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("extracts", self.extracts)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_object_value("resourceMetadata", self.resource_metadata)
        writer.write_enum_value("resourceType", self.resource_type)
        writer.write_object_value("sensitivityLabel", self.sensitivity_label)
        writer.write_str_value("webUrl", self.web_url)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from ...models.data_source_configuration import DataSourceConfiguration
    from ...models.retrieval_data_source import RetrievalDataSource

@dataclass
class RetrievalPostRequestBody(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The dataSource property
    data_source: Optional[RetrievalDataSource] = None
    # The dataSourceConfiguration property
    data_source_configuration: Optional[DataSourceConfiguration] = None
    # The filterExpression property
    filter_expression: Optional[str] = None
    # The maximumNumberOfResults property
    maximum_number_of_results: Optional[int] = None
    # The queryString property
    query_string: Optional[str] = None
    # The resourceMetadata property
    resource_metadata: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RetrievalPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RetrievalPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RetrievalPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from ...models.data_source_configuration import DataSourceConfiguration
        from ...models.retrieval_data_source import RetrievalDataSource

        from ...models.data_source_configuration import DataSourceConfiguration
        from ...models.retrieval_data_source import RetrievalDataSource

        fields: dict[str, Callable[[Any], None]] = {
            "dataSource": lambda n : setattr(self, 'data_source', n.get_enum_value(RetrievalDataSource)),
            "dataSourceConfiguration": lambda n : setattr(self, 'data_source_configuration', n.get_object_value(DataSourceConfiguration)),
            "filterExpression": lambda n : setattr(self, 'filter_expression', n.get_str_value()),
            "maximumNumberOfResults": lambda n : setattr(self, 'maximum_number_of_results', n.get_int_value()),
            "queryString": lambda n : setattr(self, 'query_string', n.get_str_value()),
            "resourceMetadata": lambda n : setattr(self, 'resource_metadata', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("dataSource", self.data_source)
        writer.write_object_value("dataSourceConfiguration", self.data_source_configuration)
        writer.write_str_value("filterExpression", self.filter_expression)
        writer.write_int_value("maximumNumberOfResults", self.maximum_number_of_results)
        writer.write_str_value("queryString", self.query_string)
        writer.write_collection_of_primitive_values("resourceMetadata", self.resource_metadata)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .endpoint_type import EndpointType
    from .identity import Identity
    from .identity_set import IdentitySet

from .identity_set import IdentitySet

@dataclass
class CommunicationsIdentitySet(IdentitySet, Parsable):
    # The OdataType property
    odata_type: Optional[str] = "#microsoft.graph.communicationsIdentitySet"
    # The application instance associated with this action.
    application_instance: Optional[Identity] = None
    # An identity the participant would like to present itself as to the other participants in the call.
    asserted_identity: Optional[Identity] = None
    # The Azure Communication Services user associated with this action.
    azure_communication_services_user: Optional[Identity] = None
    # The encrypted user associated with this action.
    encrypted: Optional[Identity] = None
    # Type of endpoint that the participant uses. Possible values are: default, voicemail, skypeForBusiness, skypeForBusinessVoipPhone, unknownFutureValue.
    endpoint_type: Optional[EndpointType] = None
    # The guest user associated with this action.
    guest: Optional[Identity] = None
    # The Skype for Business on-premises user associated with this action.
    on_premises: Optional[Identity] = None
    # The phone user associated with this action.
    phone: Optional[Identity] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CommunicationsIdentitySet:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CommunicationsIdentitySet
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CommunicationsIdentitySet()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .endpoint_type import EndpointType
        from .identity import Identity
        from .identity_set import IdentitySet

        from .endpoint_type import EndpointType
        from .identity import Identity
        from .identity_set import IdentitySet

        fields: dict[str, Callable[[Any], None]] = {
            "applicationInstance": lambda n : setattr(self, 'application_instance', n.get_object_value(Identity)),
            "assertedIdentity": lambda n : setattr(self, 'asserted_identity', n.get_object_value(Identity)),
            "azureCommunicationServicesUser": lambda n : setattr(self, 'azure_communication_services_user', n.get_object_value(Identity)),
            "encrypted": lambda n : setattr(self, 'encrypted', n.get_object_value(Identity)),
            "endpointType": lambda n : setattr(self, 'endpoint_type', n.get_enum_value(EndpointType)),
            "guest": lambda n : setattr(self, 'guest', n.get_object_value(Identity)),
            "onPremises": lambda n : setattr(self, 'on_premises', n.get_object_value(Identity)),
            "phone": lambda n : setattr(self, 'phone', n.get_object_value(Identity)),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_object_value("applicationInstance", self.application_instance)
        writer.write_object_value("assertedIdentity", self.asserted_identity)
        writer.write_object_value("azureCommunicationServicesUser", self.azure_communication_services_user)
        writer.write_object_value("encrypted", self.encrypted)
        writer.write_enum_value("endpointType", self.endpoint_type)
        writer.write_object_value("guest", self.guest)
        writer.write_object_value("onPremises", self.on_premises)
        writer.write_object_value("phone", self.phone)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .retrieval_hit import RetrievalHit

@dataclass
class RetrievalResponse(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The OdataType property
    odata_type: Optional[str] = None
    # The retrievalHits property
    retrieval_hits: Optional[list[RetrievalHit]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RetrievalResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RetrievalResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RetrievalResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .retrieval_hit import RetrievalHit

        from .retrieval_hit import RetrievalHit

        fields: dict[str, Callable[[Any], None]] = {
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
            "retrievalHits": lambda n : setattr(self, 'retrieval_hits', n.get_collection_of_object_values(RetrievalHit)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_collection_of_object_values("retrievalHits", self.retrieval_hits)
        writer.write_additional_data_value(self.additional_data)
    




from enum import Enum

class RetrievalDataSource(str, Enum):
    SharePoint = "sharePoint",
    OneDriveBusiness = "oneDriveBusiness",
    ExternalItem = "externalItem",
    UnknownFutureValue = "unknownFutureValue",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.api_error import APIError
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .main_error import MainError

@dataclass
class ODataError(APIError, AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The error property
    error: Optional[MainError] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ODataError:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ODataError
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ODataError()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .main_error import MainError

        from .main_error import MainError

        fields: dict[str, Callable[[Any], None]] = {
            "error": lambda n : setattr(self, 'error', n.get_object_value(MainError)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("error", self.error)
        writer.write_additional_data_value(self.additional_data)
    
    @property
    def primary_message(self) -> Optional[str]:
        """
        The primary error message.
        """
        if self.error is not None:
            return '' if self.error.message is None else self.error.message
        return ''




# ------------------------------------
# Copyright (c) Microsoft Corporation. All Rights Reserved.
# Licensed under the MIT License.
# See License in the project root for license information.
# -----------------------------------

from __future__ import annotations

from typing import TYPE_CHECKING, List, Optional, Union

from azure.core.credentials import TokenCredential
from azure.core.credentials_async import AsyncTokenCredential
from kiota_authentication_azure.azure_identity_authentication_provider import (
    AzureIdentityAuthenticationProvider,
)

from .agents_m365_copilot_request_adapter import AgentsM365CopilotRequestAdapter
from .generated.base_agents_m365_copilot_service_client import BaseAgentsM365CopilotServiceClient

if TYPE_CHECKING:
    from microsoft_agents_m365copilot_core import BatchRequestBuilder

    from .generated.copilot.users.users_request_builder import AiUserItemRequestBuilder

class AgentsM365CopilotServiceClient(BaseAgentsM365CopilotServiceClient):

    def __init__(
        self,
        credentials: Optional[Union[TokenCredential, AsyncTokenCredential]] = None,
        scopes: Optional[List[str]] = None,
        request_adapter: Optional[AgentsM365CopilotRequestAdapter] = None,
    ) -> None:
        """Constructs a client instance to use for making requests.

        Args:
            credentials (Union[TokenCredential, AsyncTokenCredential]): The
            tokenCredential to use for authentication. 
            scopes (Optional[List[str]]): The scopes to use for authentication.
            Defaults to ['https://graph.microsoft.com/.default'].
            request_adapter (Optional[AgentsM365CopilotRequestAdapter], optional): The request
            adapter to use for requests. Defaults to None.
        """

        if not request_adapter:
            if not credentials:
                raise ValueError("Missing request adapter or valid credentials")

            if scopes:
                auth_provider = AzureIdentityAuthenticationProvider(credentials, scopes=scopes)
            else:
                auth_provider = AzureIdentityAuthenticationProvider(credentials)

            request_adapter = AgentsM365CopilotRequestAdapter(auth_provider)

        super().__init__(request_adapter)



from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .identity import Identity
    from .teamwork_application_identity_type import TeamworkApplicationIdentityType

from .identity import Identity

@dataclass
class TeamworkApplicationIdentity(Identity, Parsable):
    # The OdataType property
    odata_type: Optional[str] = "#microsoft.graph.teamworkApplicationIdentity"
    # Type of application that is referenced. Possible values are: aadApplication, bot, tenantBot, office365Connector, outgoingWebhook, and unknownFutureValue.
    application_identity_type: Optional[TeamworkApplicationIdentityType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TeamworkApplicationIdentity:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TeamworkApplicationIdentity
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TeamworkApplicationIdentity()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .identity import Identity
        from .teamwork_application_identity_type import TeamworkApplicationIdentityType

        from .identity import Identity
        from .teamwork_application_identity_type import TeamworkApplicationIdentityType

        fields: dict[str, Callable[[Any], None]] = {
            "applicationIdentityType": lambda n : setattr(self, 'application_identity_type', n.get_enum_value(TeamworkApplicationIdentityType)),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_enum_value("applicationIdentityType", self.application_identity_type)
    




# ------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  
# Licensed under the MIT License.  See License in the project root for license information.
# ------------------------------------------------------------------------------

"""
Python client for the Microsoft CCS Python SDK.
"""
from ._version import VERSION
from .agents_m365_copilot_request_adapter import AgentsM365CopilotRequestAdapter
from .agents_m365_copilot_service_client import AgentsM365CopilotServiceClient

__version__ = VERSION

__all__ = [
    "AgentsM365CopilotServiceClient", "AgentsM365CopilotRequestAdapter"
]



from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .error_details import ErrorDetails
    from .inner_error import InnerError

@dataclass
class MainError(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The code property
    code: Optional[str] = None
    # The details property
    details: Optional[list[ErrorDetails]] = None
    # The innerError property
    inner_error: Optional[InnerError] = None
    # The message property
    message: Optional[str] = None
    # The target property
    target: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MainError:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MainError
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MainError()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .error_details import ErrorDetails
        from .inner_error import InnerError

        from .error_details import ErrorDetails
        from .inner_error import InnerError

        fields: dict[str, Callable[[Any], None]] = {
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "details": lambda n : setattr(self, 'details', n.get_collection_of_object_values(ErrorDetails)),
            "innerError": lambda n : setattr(self, 'inner_error', n.get_object_value(InnerError)),
            "message": lambda n : setattr(self, 'message', n.get_str_value()),
            "target": lambda n : setattr(self, 'target', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("code", self.code)
        writer.write_collection_of_object_values("details", self.details)
        writer.write_object_value("innerError", self.inner_error)
        writer.write_str_value("message", self.message)
        writer.write_str_value("target", self.target)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .ai_interaction import AiInteraction
    from .ai_interaction_history import AiInteractionHistory
    from .ai_user import AiUser
    from .copilot_admin import CopilotAdmin
    from .copilot_admin_limited_mode import CopilotAdminLimitedMode
    from .copilot_admin_setting import CopilotAdminSetting
    from .copilot_report_root import CopilotReportRoot

@dataclass
class Entity(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The unique identifier for an entity. Read-only.
    id: Optional[str] = None
    # The OdataType property
    odata_type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Entity:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Entity
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        try:
            child_node = parse_node.get_child_node("@odata.type")
            mapping_value = child_node.get_str_value() if child_node else None
        except AttributeError:
            mapping_value = None
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.aiInteraction".casefold():
            from .ai_interaction import AiInteraction

            return AiInteraction()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.aiInteractionHistory".casefold():
            from .ai_interaction_history import AiInteractionHistory

            return AiInteractionHistory()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.aiUser".casefold():
            from .ai_user import AiUser

            return AiUser()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.copilotAdmin".casefold():
            from .copilot_admin import CopilotAdmin

            return CopilotAdmin()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.copilotAdminLimitedMode".casefold():
            from .copilot_admin_limited_mode import CopilotAdminLimitedMode

            return CopilotAdminLimitedMode()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.copilotAdminSetting".casefold():
            from .copilot_admin_setting import CopilotAdminSetting

            return CopilotAdminSetting()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.copilotReportRoot".casefold():
            from .copilot_report_root import CopilotReportRoot

            return CopilotReportRoot()
        return Entity()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .ai_interaction import AiInteraction
        from .ai_interaction_history import AiInteractionHistory
        from .ai_user import AiUser
        from .copilot_admin import CopilotAdmin
        from .copilot_admin_limited_mode import CopilotAdminLimitedMode
        from .copilot_admin_setting import CopilotAdminSetting
        from .copilot_report_root import CopilotReportRoot

        from .ai_interaction import AiInteraction
        from .ai_interaction_history import AiInteractionHistory
        from .ai_user import AiUser
        from .copilot_admin import CopilotAdmin
        from .copilot_admin_limited_mode import CopilotAdminLimitedMode
        from .copilot_admin_setting import CopilotAdminSetting
        from .copilot_report_root import CopilotReportRoot

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("id", self.id)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .admin.admin_request_builder import AdminRequestBuilder
    from .interaction_history.interaction_history_request_builder import InteractionHistoryRequestBuilder
    from .reports.reports_request_builder import ReportsRequestBuilder
    from .retrieval.retrieval_request_builder import RetrievalRequestBuilder
    from .users.users_request_builder import UsersRequestBuilder

class CopilotRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /copilot
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new CopilotRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/copilot", path_parameters)
    
    @property
    def admin(self) -> AdminRequestBuilder:
        """
        Provides operations to manage the admin property of the microsoft.graph.copilotRoot entity.
        """
        from .admin.admin_request_builder import AdminRequestBuilder

        return AdminRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def interaction_history(self) -> InteractionHistoryRequestBuilder:
        """
        Provides operations to manage the interactionHistory property of the microsoft.graph.copilotRoot entity.
        """
        from .interaction_history.interaction_history_request_builder import InteractionHistoryRequestBuilder

        return InteractionHistoryRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def reports(self) -> ReportsRequestBuilder:
        """
        Provides operations to manage the reports property of the microsoft.graph.copilotRoot entity.
        """
        from .reports.reports_request_builder import ReportsRequestBuilder

        return ReportsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def retrieval(self) -> RetrievalRequestBuilder:
        """
        Provides operations to call the retrieval method.
        """
        from .retrieval.retrieval_request_builder import RetrievalRequestBuilder

        return RetrievalRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def users(self) -> UsersRequestBuilder:
        """
        Provides operations to manage the users property of the microsoft.graph.copilotRoot entity.
        """
        from .users.users_request_builder import UsersRequestBuilder

        return UsersRequestBuilder(self.request_adapter, self.path_parameters)
    




from enum import Enum

class RetrievalEntityType(str, Enum):
    Site = "site",
    List_ = "list",
    ListItem = "listItem",
    Drive = "drive",
    DriveItem = "driveItem",
    ExternalItem = "externalItem",
    UnknownFutureValue = "unknownFutureValue",




from typing import Optional

import httpx
from kiota_abstractions.authentication import AuthenticationProvider
from kiota_http.middleware.options import UrlReplaceHandlerOption
from microsoft_agents_m365copilot_core import (
    APIVersion,
    BaseMicrosoftAgentsM365CopilotRequestAdapter,
    MicrosoftAgentsM365CopilotClientFactory,
    MicrosoftAgentsM365CopilotTelemetryHandlerOption,
)

from ._version import VERSION

options = {
    MicrosoftAgentsM365CopilotTelemetryHandlerOption.get_key(): MicrosoftAgentsM365CopilotTelemetryHandlerOption(
        api_version=APIVersion.v1,
        sdk_version=VERSION)
}


class AgentsM365CopilotRequestAdapter(BaseMicrosoftAgentsM365CopilotRequestAdapter):
    def __init__(self, auth_provider: AuthenticationProvider,
                 client: Optional[httpx.AsyncClient] = None) -> None:
        if client is None:
            client = MicrosoftAgentsM365CopilotClientFactory.create_with_default_middleware(options=options)
        super().__init__(auth_provider, http_client=client)



from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .azure_communication_services_user_identity import AzureCommunicationServicesUserIdentity
    from .call_records.user_identity import UserIdentity
    from .communications_application_identity import CommunicationsApplicationIdentity
    from .communications_application_instance_identity import CommunicationsApplicationInstanceIdentity
    from .communications_encrypted_identity import CommunicationsEncryptedIdentity
    from .communications_guest_identity import CommunicationsGuestIdentity
    from .communications_phone_identity import CommunicationsPhoneIdentity
    from .communications_user_identity import CommunicationsUserIdentity
    from .email_identity import EmailIdentity
    from .initiator import Initiator
    from .provisioned_identity import ProvisionedIdentity
    from .provisioning_service_principal import ProvisioningServicePrincipal
    from .provisioning_system import ProvisioningSystem
    from .service_principal_identity import ServicePrincipalIdentity
    from .share_point_identity import SharePointIdentity
    from .teamwork_application_identity import TeamworkApplicationIdentity
    from .teamwork_conversation_identity import TeamworkConversationIdentity
    from .teamwork_tag_identity import TeamworkTagIdentity
    from .teamwork_user_identity import TeamworkUserIdentity
    from .user_identity import UserIdentity

@dataclass
class Identity(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The display name of the identity.For drive items, the display name might not always be available or up to date. For example, if a user changes their display name the API might show the new value in a future response, but the items associated with the user don't show up as changed when using delta.
    display_name: Optional[str] = None
    # Unique identifier for the identity or actor. For example, in the access reviews decisions API, this property might record the id of the principal, that is, the group, user, or application that's subject to review.
    id: Optional[str] = None
    # The OdataType property
    odata_type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Identity:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Identity
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        try:
            child_node = parse_node.get_child_node("@odata.type")
            mapping_value = child_node.get_str_value() if child_node else None
        except AttributeError:
            mapping_value = None
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.azureCommunicationServicesUserIdentity".casefold():
            from .azure_communication_services_user_identity import AzureCommunicationServicesUserIdentity

            return AzureCommunicationServicesUserIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.callRecords.userIdentity".casefold():
            from .call_records.user_identity import UserIdentity
            from .user_identity import UserIdentity

            return UserIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.communicationsApplicationIdentity".casefold():
            from .communications_application_identity import CommunicationsApplicationIdentity

            return CommunicationsApplicationIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.communicationsApplicationInstanceIdentity".casefold():
            from .communications_application_instance_identity import CommunicationsApplicationInstanceIdentity

            return CommunicationsApplicationInstanceIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.communicationsEncryptedIdentity".casefold():
            from .communications_encrypted_identity import CommunicationsEncryptedIdentity

            return CommunicationsEncryptedIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.communicationsGuestIdentity".casefold():
            from .communications_guest_identity import CommunicationsGuestIdentity

            return CommunicationsGuestIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.communicationsPhoneIdentity".casefold():
            from .communications_phone_identity import CommunicationsPhoneIdentity

            return CommunicationsPhoneIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.communicationsUserIdentity".casefold():
            from .communications_user_identity import CommunicationsUserIdentity

            return CommunicationsUserIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.emailIdentity".casefold():
            from .email_identity import EmailIdentity

            return EmailIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.initiator".casefold():
            from .initiator import Initiator

            return Initiator()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.provisionedIdentity".casefold():
            from .provisioned_identity import ProvisionedIdentity

            return ProvisionedIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.provisioningServicePrincipal".casefold():
            from .provisioning_service_principal import ProvisioningServicePrincipal

            return ProvisioningServicePrincipal()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.provisioningSystem".casefold():
            from .provisioning_system import ProvisioningSystem

            return ProvisioningSystem()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.servicePrincipalIdentity".casefold():
            from .service_principal_identity import ServicePrincipalIdentity

            return ServicePrincipalIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.sharePointIdentity".casefold():
            from .share_point_identity import SharePointIdentity

            return SharePointIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.teamworkApplicationIdentity".casefold():
            from .teamwork_application_identity import TeamworkApplicationIdentity

            return TeamworkApplicationIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.teamworkConversationIdentity".casefold():
            from .teamwork_conversation_identity import TeamworkConversationIdentity

            return TeamworkConversationIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.teamworkTagIdentity".casefold():
            from .teamwork_tag_identity import TeamworkTagIdentity

            return TeamworkTagIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.teamworkUserIdentity".casefold():
            from .teamwork_user_identity import TeamworkUserIdentity

            return TeamworkUserIdentity()
        if mapping_value and mapping_value.casefold() == "#microsoft.graph.userIdentity".casefold():
            from .call_records.user_identity import UserIdentity
            from .user_identity import UserIdentity

            return UserIdentity()
        return Identity()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .azure_communication_services_user_identity import AzureCommunicationServicesUserIdentity
        from .call_records.user_identity import UserIdentity
        from .communications_application_identity import CommunicationsApplicationIdentity
        from .communications_application_instance_identity import CommunicationsApplicationInstanceIdentity
        from .communications_encrypted_identity import CommunicationsEncryptedIdentity
        from .communications_guest_identity import CommunicationsGuestIdentity
        from .communications_phone_identity import CommunicationsPhoneIdentity
        from .communications_user_identity import CommunicationsUserIdentity
        from .email_identity import EmailIdentity
        from .initiator import Initiator
        from .provisioned_identity import ProvisionedIdentity
        from .provisioning_service_principal import ProvisioningServicePrincipal
        from .provisioning_system import ProvisioningSystem
        from .service_principal_identity import ServicePrincipalIdentity
        from .share_point_identity import SharePointIdentity
        from .teamwork_application_identity import TeamworkApplicationIdentity
        from .teamwork_conversation_identity import TeamworkConversationIdentity
        from .teamwork_tag_identity import TeamworkTagIdentity
        from .teamwork_user_identity import TeamworkUserIdentity
        from .user_identity import UserIdentity

        from .azure_communication_services_user_identity import AzureCommunicationServicesUserIdentity
        from .call_records.user_identity import UserIdentity
        from .communications_application_identity import CommunicationsApplicationIdentity
        from .communications_application_instance_identity import CommunicationsApplicationInstanceIdentity
        from .communications_encrypted_identity import CommunicationsEncryptedIdentity
        from .communications_guest_identity import CommunicationsGuestIdentity
        from .communications_phone_identity import CommunicationsPhoneIdentity
        from .communications_user_identity import CommunicationsUserIdentity
        from .email_identity import EmailIdentity
        from .initiator import Initiator
        from .provisioned_identity import ProvisionedIdentity
        from .provisioning_service_principal import ProvisioningServicePrincipal
        from .provisioning_system import ProvisioningSystem
        from .service_principal_identity import ServicePrincipalIdentity
        from .share_point_identity import SharePointIdentity
        from .teamwork_application_identity import TeamworkApplicationIdentity
        from .teamwork_conversation_identity import TeamworkConversationIdentity
        from .teamwork_tag_identity import TeamworkTagIdentity
        from .teamwork_user_identity import TeamworkUserIdentity
        from .user_identity import UserIdentity

        fields: dict[str, Callable[[Any], None]] = {
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("displayName", self.display_name)
        writer.write_str_value("id", self.id)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .ai_interaction_attachment import AiInteractionAttachment
    from .ai_interaction_context import AiInteractionContext
    from .ai_interaction_link import AiInteractionLink
    from .ai_interaction_mention import AiInteractionMention
    from .ai_interaction_type import AiInteractionType
    from .entity import Entity
    from .identity_set import IdentitySet
    from .item_body import ItemBody

from .entity import Entity

@dataclass
class AiInteraction(Entity, Parsable):
    # The appClass property
    app_class: Optional[str] = None
    # The attachments property
    attachments: Optional[list[AiInteractionAttachment]] = None
    # The body property
    body: Optional[ItemBody] = None
    # The contexts property
    contexts: Optional[list[AiInteractionContext]] = None
    # The conversationType property
    conversation_type: Optional[str] = None
    # The createdDateTime property
    created_date_time: Optional[datetime.datetime] = None
    # The etag property
    etag: Optional[str] = None
    # The from property
    from_: Optional[IdentitySet] = None
    # The interactionType property
    interaction_type: Optional[AiInteractionType] = None
    # The links property
    links: Optional[list[AiInteractionLink]] = None
    # The locale property
    locale: Optional[str] = None
    # The mentions property
    mentions: Optional[list[AiInteractionMention]] = None
    # The OdataType property
    odata_type: Optional[str] = None
    # The requestId property
    request_id: Optional[str] = None
    # The sessionId property
    session_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AiInteraction:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AiInteraction
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AiInteraction()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .ai_interaction_attachment import AiInteractionAttachment
        from .ai_interaction_context import AiInteractionContext
        from .ai_interaction_link import AiInteractionLink
        from .ai_interaction_mention import AiInteractionMention
        from .ai_interaction_type import AiInteractionType
        from .entity import Entity
        from .identity_set import IdentitySet
        from .item_body import ItemBody

        from .ai_interaction_attachment import AiInteractionAttachment
        from .ai_interaction_context import AiInteractionContext
        from .ai_interaction_link import AiInteractionLink
        from .ai_interaction_mention import AiInteractionMention
        from .ai_interaction_type import AiInteractionType
        from .entity import Entity
        from .identity_set import IdentitySet
        from .item_body import ItemBody

        fields: dict[str, Callable[[Any], None]] = {
            "appClass": lambda n : setattr(self, 'app_class', n.get_str_value()),
            "attachments": lambda n : setattr(self, 'attachments', n.get_collection_of_object_values(AiInteractionAttachment)),
            "body": lambda n : setattr(self, 'body', n.get_object_value(ItemBody)),
            "contexts": lambda n : setattr(self, 'contexts', n.get_collection_of_object_values(AiInteractionContext)),
            "conversationType": lambda n : setattr(self, 'conversation_type', n.get_str_value()),
            "createdDateTime": lambda n : setattr(self, 'created_date_time', n.get_datetime_value()),
            "etag": lambda n : setattr(self, 'etag', n.get_str_value()),
            "from": lambda n : setattr(self, 'from_', n.get_object_value(IdentitySet)),
            "interactionType": lambda n : setattr(self, 'interaction_type', n.get_enum_value(AiInteractionType)),
            "links": lambda n : setattr(self, 'links', n.get_collection_of_object_values(AiInteractionLink)),
            "locale": lambda n : setattr(self, 'locale', n.get_str_value()),
            "mentions": lambda n : setattr(self, 'mentions', n.get_collection_of_object_values(AiInteractionMention)),
            "requestId": lambda n : setattr(self, 'request_id', n.get_str_value()),
            "sessionId": lambda n : setattr(self, 'session_id', n.get_str_value()),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_str_value("appClass", self.app_class)
        writer.write_collection_of_object_values("attachments", self.attachments)
        writer.write_object_value("body", self.body)
        writer.write_collection_of_object_values("contexts", self.contexts)
        writer.write_str_value("conversationType", self.conversation_type)
        writer.write_datetime_value("createdDateTime", self.created_date_time)
        writer.write_str_value("etag", self.etag)
        writer.write_object_value("from", self.from_)
        writer.write_enum_value("interactionType", self.interaction_type)
        writer.write_collection_of_object_values("links", self.links)
        writer.write_str_value("locale", self.locale)
        writer.write_collection_of_object_values("mentions", self.mentions)
        writer.write_str_value("requestId", self.request_id)
        writer.write_str_value("sessionId", self.session_id)
    




# ------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  
# Licensed under the MIT License.  See License in the project root for license information.
# ------------------------------------------------------------------------------

# The SDK version
# x-release-please-start-version
VERSION: str = '1.2.0'
# x-release-please-end
